==> trilinos_9.0.3.dfsg.orig.tar.gz: trilinos-9.0.3.dfsg.orig/packages/isorropia/src/epetra/Isorropia_EpetraColorer.cpp <==
//@HEADER
/*
************************************************************************

              Isorropia: Partitioning and Load Balancing Package
                Copyright (2006) Sandia Corporation

Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
license for use of this work by or on behalf of the U.S. Government.

This library is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as
published by the Free Software Foundation; either version 2.1 of the
License, or (at your option) any later version.

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
USA

************************************************************************
*/
//@HEADER

#include <Isorropia_EpetraColorer.hpp>
#ifdef HAVE_ISORROPIA_ZOLTAN
#include <Isorropia_EpetraZoltanLib.hpp>
#endif /* HAVE_ISORROPIA_ZOLTAN */
#include <Isorropia_Exception.hpp>
#include <Isorropia_Epetra.hpp>
#include <Isorropia_EpetraCostDescriber.hpp>

#include <Teuchos_RCP.hpp>
#include <Teuchos_ParameterList.hpp>

#ifdef HAVE_EPETRA
#include <Epetra_Comm.h>
#include <Epetra_Map.h>
#include <Epetra_Import.h>
#include <Epetra_Vector.h>
#include <Epetra_MultiVector.h>
#include <Epetra_CrsGraph.h>
#include <Epetra_CrsMatrix.h>
#include <Epetra_LinearProblem.h>
#endif

#include <cstring>
#include <iostream>
#include <sstream>
#include <string>
#include <ctype.h>

namespace Isorropia {

#ifdef HAVE_EPETRA

namespace Epetra {


Colorer::Colorer(Teuchos::RCP<const Epetra_CrsGraph> input_graph,
		 const Teuchos::ParameterList& paramlist,
		 bool compute_now):
  Operator (input_graph, paramlist) {
#ifdef HAVE_ISORROPIA_ZOLTAN
  lib_ = Teuchos::rcp(new ZoltanLibClass(input_graph));
  lib_->setInputType("GRAPH");
#else /* HAVE_ISORROPIA_ZOLTAN */
  throw Isorropia::Exception("Coloring only available in Zoltan");
  return ;
#endif /* HAVE_ISORROPIA_ZOLTAN */
  if (compute_now)
    color(true);
}

Colorer::Colorer(Teuchos::RCP<const Epetra_RowMatrix> input_matrix,
		 const Teuchos::ParameterList& paramlist,
		 bool compute_now):
  Operator (input_matrix, paramlist) {

#ifdef HAVE_ISORROPIA_ZOLTAN
  lib_ = Teuchos::rcp(new ZoltanLibClass(input_matrix));
  lib_->setInputType("GRAPH");
#else /* HAVE_ISORROPIA_ZOLTAN */
  throw Isorropia::Exception("Coloring only available in Zoltan");
  return ;
#endif /* HAVE_ISORROPIA_ZOLTAN */

  if (compute_now)
    color(true);
}

void
Colorer::color(bool force_coloring)
{
  if (alreadyComputed() && !force_coloring)
    return;

  std::string zoltan("ZOLTAN");
  Teuchos::ParameterList sublist = paramlist_.sublist(zoltan);

  lib_->color(sublist, properties_);
  operation_already_computed_ = true;
  computeNumberOfProperties();
}

#ifdef HAVE_EPETRAEXT

Teuchos::RCP<Epetra_MapColoring>
Colorer::generateMapColoring()
{
  Teuchos::RCP<Epetra_MapColoring> colorMap;

  color(false);
  colorMap = Teuchos::rcp(new Epetra_MapColoring(*input_map_));

  for(unsigned int i = 0; i < properties_.size(); i++ ) {
    (*colorMap)[i] = properties_[i];
  }
  return (colorMap);
}

#endif /* HAVE_EPETRAEXT */

} // namespace EPETRA

#endif //HAVE_EPETRA

}//namespace Isorropia

==> trilinos_10.0.4.dfsg.orig.tar.gz: trilinos-10.0.4.dfsg.orig/packages/isorropia/src/epetra/Isorropia_EpetraColorer.cpp <==
//@HEADER
/*
************************************************************************

              Isorropia: Partitioning and Load Balancing Package
                Copyright (2006) Sandia Corporation

Under terms of Contract DE-AC04-94AL85000, there is a non-exclusive
license for use of this work by or on behalf of the U.S. Government.

This library is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as
published by the Free Software Foundation; either version 2.1 of the
License, or (at your option) any later version.

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
USA

************************************************************************
*/
//@HEADER

#include <Isorropia_EpetraColorer.hpp>
#ifdef HAVE_ISORROPIA_ZOLTAN
#include <Isorropia_EpetraZoltanLib.hpp>
#endif /* HAVE_ISORROPIA_ZOLTAN */
#include <Isorropia_Exception.hpp>
#include <Isorropia_Epetra.hpp>
#include <Isorropia_EpetraCostDescriber.hpp>

#include <Teuchos_RCP.hpp>
#include <Teuchos_ParameterList.hpp>

#ifdef HAVE_EPETRA
#include <Epetra_Comm.h>
#include <Epetra_Map.h>
#include <Epetra_Import.h>
#include <Epetra_Vector.h>
#include <Epetra_MultiVector.h>
#include <Epetra_CrsGraph.h>
#include <Epetra_CrsMatrix.h>
#include <Epetra_LinearProblem.h>
#endif

#include <cstring>
#include <iostream>
#include <sstream>
#include <string>
#include <ctype.h>

namespace Isorropia {

#ifdef HAVE_EPETRA

namespace Epetra {


Colorer::Colorer(Teuchos::RCP<const Epetra_CrsGraph> input_graph,
		 const Teuchos::ParameterList& paramlist,
		 bool compute_now):
  Operator(input_graph, paramlist, 1){

#ifdef HAVE_EPETRAEXT
  colmap_ = Teuchos::rcp(&(input_graph->ColMap()),false);
#endif /* HAVE_EPETRAEXT */

#ifdef HAVE_ISORROPIA_ZOLTAN
  lib_ = Teuchos::rcp(new ZoltanLibClass(input_graph, Library::graph_input_));
#else /* HAVE_ISORROPIA_ZOLTAN */
  throw Isorropia::Exception("Coloring only available in Zoltan");
  return ;
#endif /* HAVE_ISORROPIA_ZOLTAN */
  if (compute_now)
    color(true);
}

Colorer::Colorer(Teuchos::RCP<const Epetra_RowMatrix> input_matrix,
		 const Teuchos::ParameterList& paramlist,
		 bool compute_now):
  Operator (input_matrix, paramlist, 1) {

#ifdef HAVE_EPETRAEXT
  colmap_ = Teuchos::rcp(&(input_matrix->RowMatrixColMap()),false);
#endif /* HAVE_EPETRAEXT */

#ifdef HAVE_ISORROPIA_ZOLTAN
  lib_ = Teuchos::rcp(new ZoltanLibClass(input_matrix, Library::graph_input_));
#else /* HAVE_ISORROPIA_ZOLTAN */
  throw Isorropia::Exception("Coloring only available in Zoltan");
  return ;
#endif /* HAVE_ISORROPIA_ZOLTAN */

  if (compute_now)
    color(true);
}

void
Colorer::color(bool force_coloring)
{
  if (alreadyComputed() && !force_coloring)
    return;

  std::string zoltan("ZOLTAN");
  Teuchos::ParameterList sublist = paramlist_.sublist(zoltan);

  if (paramlist_.isParameter("DISTANCE")) {
    sublist.set("COLORING_PROBLEM", "DISTANCE-"+paramlist_.get<std::string>("DISTANCE"));
  }

  lib_->color(sublist, properties_);
  operation_already_computed_ = true;
  computeNumberOfProperties();
}

#ifdef HAVE_EPETRAEXT

Teuchos::RCP<Epetra_MapColoring>
Colorer::generateRowMapColoring()
{
  Teuchos::RCP<Epetra_MapColoring> colorMap;

  color(false);
  colorMap = Teuchos::rcp(new Epetra_MapColoring(*input_map_, 1));

  for(unsigned int i = 0; i < properties_.size(); i++ ) {
    (*colorMap)[i] = properties_[i];
  }
  return (colorMap);
}


Teuchos::RCP<Epetra_MapColoring>
Colorer::generateColMapColoring()
{
  Teuchos::RCP<Epetra_MapColoring> rowColorMap = generateRowMapColoring();

  // Color map has colored rows -- need colored columns
  Epetra_Import importer(*colmap_, *input_map_);

  Teuchos::RCP<Epetra_MapColoring> colorMap =
    Teuchos::rcp(new Epetra_MapColoring(*colmap_));

  colorMap->Import(*rowColorMap, importer, Insert);
  return (colorMap);
}


#endif /* HAVE_EPETRAEXT */

} // namespace EPETRA

#endif //HAVE_EPETRA

}//namespace Isorropia
